@@ -1,8 +1,6 @@
 from __future__ import division, absolute_import, unicode_literals
-
 import os
 import re
-import subprocess
 import sys
 from fnmatch import fnmatch
 from io import StringIO
@@ -12,22 +10,21 @@ try:
 except ImportError:
     send2trash = None
 
-from cola import compat
-from cola import core
-from cola import fsmonitor
-from cola import gitcfg
-from cola import gitcmds
-from cola import icons
-from cola import utils
-from cola import difftool
-from cola import resources
-from cola.diffparse import DiffParser
-from cola.git import STDOUT
-from cola.i18n import N_
-from cola.interaction import Interaction
-from cola.models import main
-from cola.models import prefs
-from cola.models import selection
+from . import compat
+from . import core
+from . import fsmonitor
+from . import gitcfg
+from . import gitcmds
+from . import icons
+from . import utils
+from . import resources
+from .diffparse import DiffParser
+from .git import STDOUT
+from .i18n import N_
+from .interaction import Interaction
+from .models import main
+from .models import prefs
+from .models import selection
 
 
 class UsageError(Exception):
@@ -43,8 +40,10 @@ class BaseCommand(object):
 
     DISABLED = False
 
-    def __init__(self):
+    def __init__(self, **kwargs):
         self.undoable = False
+        for k, v in kwargs.items():
+            setattr(self, k, v)
 
     def is_undoable(self):
         """Can this be undone?"""
@@ -63,8 +62,8 @@ class BaseCommand(object):
 
 class ConfirmAction(BaseCommand):
 
-    def __init__(self):
-        BaseCommand.__init__(self)
+    def __init__(self, **kwargs):
+        BaseCommand.__init__(self, **kwargs)
 
     def ok_to_run(self):
         return True
@@ -109,9 +108,12 @@ class ConfirmAction(BaseCommand):
 class ModelCommand(BaseCommand):
     """Commands that manipulate the main models"""
 
-    def __init__(self):
-        BaseCommand.__init__(self)
+    def __init__(self, **kwargs):
+        # Note: self.model is set before calling the base class constructor
+        # to allow being having the `model` value be overridden by passing
+        # `model=xxx` during construction.
         self.model = main.model()
+        BaseCommand.__init__(self, **kwargs)
 
 
 class Command(ModelCommand):
@@ -225,16 +227,17 @@ class ApplyDiffSelection(Command):
             return
 
         cfg = gitcfg.current()
-        tmp_path = utils.tmp_filename('patch')
+        encoding = cfg.file_encoding(self.model.filename)
+        tmp_file = utils.tmp_filename('patch')
         try:
-            core.write(tmp_path, patch,
-                       encoding=cfg.file_encoding(self.model.filename))
+            core.write(tmp_file, patch, encoding=encoding)
             if self.apply_to_worktree:
-                status, out, err = self.model.apply_diff_to_worktree(tmp_path)
+                status, out, err = self.model.apply_diff_to_worktree(tmp_file)
             else:
-                status, out, err = self.model.apply_diff(tmp_path)
+                status, out, err = self.model.apply_diff(tmp_file)
         finally:
-            os.unlink(tmp_path)
+            core.unlink(tmp_file)
+
         Interaction.log_status(status, out, err)
         self.model.update_file_status(update_index=True)
 
@@ -271,8 +274,8 @@ class ApplyPatches(Command):
         basenames = '\n'.join([os.path.basename(p) for p in self.patches])
         Interaction.information(
                 N_('Patch(es) Applied'),
-                (N_('%d patch(es) applied.') + '\n\n%s') %
-                    (len(self.patches), basenames))
+                (N_('%d patch(es) applied.') +
+                 '\n\n%s') % (len(self.patches), basenames))
 
 
 class Archive(BaseCommand):
@@ -374,6 +377,55 @@ class ResetMode(Command):
         self.model.update_file_status()
 
 
+class ResetCommand(ConfirmAction):
+
+    def __init__(self, ref):
+        ConfirmAction.__init__(self, model=main.model(), ref=ref)
+
+    def action(self):
+        status, out, err = self.reset()
+        Interaction.log_status(status, out, err)
+        return status, out, err
+
+    def success(self):
+        self.model.update_file_status()
+
+    def confirm(self):
+        raise NotImplemented('confirm() must be overridden')
+
+    def reset(self):
+        raise NotImplemented('reset() must be overridden')
+
+
+class ResetBranchHead(ResetCommand):
+
+    def confirm(self):
+        title = N_('Reset Branch')
+        question = N_('Point the current branch head to a new commit?')
+        info = N_('The branch will be reset using "git reset --mixed %s"')
+        ok_btn = N_('Reset Branch')
+        info = info % self.ref
+        return Interaction.confirm(title, question, info, ok_btn)
+
+    def reset(self):
+        git = self.model.git
+        return git.reset(self.ref, '--', mixed=True)
+
+
+class ResetWorktree(ResetCommand):
+
+    def confirm(self):
+        title = N_('Reset Worktree')
+        question = N_('Reset worktree?')
+        info = N_('The worktree will be reset using "git reset --merge %s"')
+        ok_btn = N_('Reset Worktree')
+        info = info % self.ref
+        return Interaction.confirm(title, question, info, ok_btn)
+
+    def reset(self):
+        return self.model.git.reset(self.ref, '--', merge=True)
+
+
 class Commit(ResetMode):
     """Attempt to create a new commit."""
 
@@ -390,18 +442,18 @@ class Commit(ResetMode):
         # Create the commit message file
         comment_char = prefs.comment_char()
         msg = self.strip_comments(self.msg, comment_char=comment_char)
-        tmpfile = utils.tmp_filename('commit-message')
+        tmp_file = utils.tmp_filename('commit-message')
         try:
-            core.write(tmpfile, msg)
+            core.write(tmp_file, msg)
 
             # Run 'git commit'
-            status, out, err = self.model.git.commit(F=tmpfile,
+            status, out, err = self.model.git.commit(F=tmp_file,
                                                      v=True,
                                                      gpg_sign=self.sign,
                                                      amend=self.amend,
                                                      no_verify=self.no_verify)
         finally:
-            core.unlink(tmpfile)
+            core.unlink(tmp_file)
 
         if status == 0:
             ResetMode.do(self)
@@ -417,7 +469,7 @@ class Commit(ResetMode):
     def strip_comments(msg, comment_char='#'):
         # Strip off comments
         message_lines = [line for line in msg.split('\n')
-                            if not line.startswith(comment_char)]
+                         if not line.startswith(comment_char)]
         msg = '\n'.join(message_lines)
         if not msg.endswith('\n'):
             msg += '\n'
@@ -515,10 +567,11 @@ class RemoteRename(RemoteCommand):
 
 class RemoveFromSettings(ConfirmAction):
 
-    def __init__(self, settings, repo, icon=None):
+    def __init__(self, settings, repo, name, icon=None):
         ConfirmAction.__init__(self)
         self.settings = settings
         self.repo = repo
+        self.name = name
         self.icon = icon
 
     def success(self):
@@ -528,14 +581,14 @@ class RemoveFromSettings(ConfirmAction):
 class RemoveBookmark(RemoveFromSettings):
 
     def confirm(self):
-        repo = self.repo
+        name = self.name
         title = msg = N_('Delete Bookmark?')
-        info = N_('%s will be removed from your bookmarks.') % repo
+        info = N_('%s will be removed from your bookmarks.') % name
         ok_text = N_('Delete Bookmark')
         return Interaction.confirm(title, msg, info, ok_text, icon=self.icon)
 
     def action(self):
-        self.settings.remove_bookmark(self.repo)
+        self.settings.remove_bookmark(self.repo, self.name)
         return (0, '', '')
 
 
@@ -577,7 +630,7 @@ class RemoveFiles(Command):
             if filename:
                 try:
                     remove(filename)
-                    rescan=True
+                    rescan = True
                 except:
                     bad_filenames.append(filename)
 
@@ -644,9 +697,11 @@ class RenameBranch(Command):
         self.new_branch = new_branch
 
     def do(self):
-        status, out, err = self.model.rename_branch(self.branch, self.new_branch)
+        status, out, err = self.model.rename_branch(self.branch,
+                                                    self.new_branch)
         Interaction.log_status(status, out, err)
 
+
 class DeleteRemoteBranch(Command):
     """Delete a remote git branch."""
 
@@ -665,17 +720,16 @@ class DeleteRemoteBranch(Command):
             Interaction.information(
                 N_('Remote Branch Deleted'),
                 N_('"%(branch)s" has been deleted from "%(remote)s".')
-                    % dict(branch=self.branch, remote=self.remote))
+                % dict(branch=self.branch, remote=self.remote))
         else:
             command = 'git push'
             message = (N_('"%(command)s" returned exit status %(status)d') %
-                        dict(command=command, status=status))
+                       dict(command=command, status=status))
 
             Interaction.critical(N_('Error Deleting Remote Branch'),
                                  message, out + err)
 
 
-
 class Diff(Command):
     """Perform a diff and set the model's current text."""
 
@@ -706,7 +760,7 @@ class Diffstat(Command):
                                    M=True,
                                    stat=True)[STDOUT]
         self.new_diff_text = diff
-        self.new_mode = self.model.mode_worktree
+        self.new_mode = self.model.mode_diffstat
 
 
 class DiffStaged(Diff):
@@ -740,8 +794,7 @@ class Difftool(Command):
         self.filenames = filenames
 
     def do(self):
-        difftool.launch_with_head(self.filenames,
-                                  self.staged, self.model.head)
+        difftool_launch_with_head(self.filenames, self.staged, self.model.head)
 
 
 class Edit(Command):
@@ -832,7 +885,7 @@ class LaunchDifftool(BaseCommand):
                     argv.extend(mergetool)
                 core.fork(argv)
         else:
-            difftool.run()
+            difftool_run()
 
 
 class LaunchTerminal(BaseCommand):
@@ -876,7 +929,7 @@ class LoadCommitMessageFromFile(Command):
         self.old_directory = self.model.directory
 
     def do(self):
-        path = self.path
+        path = os.path.expanduser(self.path)
         if not path or not core.isfile(path):
             raise UsageError(N_('Error: Cannot find commit template'),
                              N_('%s: No such file or directory.') % path)
@@ -906,15 +959,14 @@ class LoadCommitMessageFromTemplate(LoadCommitMessageFromFile):
         return LoadCommitMessageFromFile.do(self)
 
 
-
-class LoadCommitMessageFromSHA1(Command):
+class LoadCommitMessageFromOID(Command):
     """Load a previous commit message"""
 
-    def __init__(self, sha1, prefix=''):
+    def __init__(self, oid, prefix=''):
         Command.__init__(self)
-        self.sha1 = sha1
+        self.oid = oid
         self.old_commitmsg = self.model.commitmsg
-        self.new_commitmsg = prefix + self.model.prev_commitmsg(sha1)
+        self.new_commitmsg = prefix + self.model.prev_commitmsg(oid)
         self.undoable = True
 
     def do(self):
@@ -924,11 +976,54 @@ class LoadCommitMessageFromSHA1(Command):
         self.model.set_commitmsg(self.old_commitmsg)
 
 
-class LoadFixupMessage(LoadCommitMessageFromSHA1):
+class PrepareCommitMessageHook(Command):
+    """Use the cola-prepare-commit-msg hook to prepare the commit message
+    """
+    def __init__(self):
+        Command.__init__(self)
+        self.old_commitmsg = self.model.commitmsg
+        self.undoable = True
+
+    def get_message(self):
+
+        title = N_('Error running prepare-commitmsg hook')
+        hook = gitcmds.prepare_commit_message_hook()
+
+        if os.path.exists(hook):
+            filename = self.model.save_commitmsg()
+            status, out, err = core.run_command([hook, filename])
+
+            if status == 0:
+                result = core.read(filename)
+            else:
+                result = self.old_commitmsg
+                details = out or ''
+                if err:
+                    if details and not details.endswith('\n'):
+                        details += '\n'
+                    details += err
+                message = N_('"%s" returned exit status %d') % (hook, status)
+                Interaction.critical(title, message=message, details=details)
+        else:
+            message = N_('A hook must be provided at "%s"') % hook
+            Interaction.critical(title, message=message)
+            result = self.old_commitmsg
+
+        return result
+
+    def do(self):
+        msg = self.get_message()
+        self.model.set_commitmsg(msg)
+
+    def undo(self):
+        self.model.set_commitmsg(self.old_commitmsg)
+
+
+class LoadFixupMessage(LoadCommitMessageFromOID):
     """Load a fixup message"""
 
-    def __init__(self, sha1):
-        LoadCommitMessageFromSHA1.__init__(self, sha1, prefix='fixup! ')
+    def __init__(self, oid):
+        LoadCommitMessageFromOID.__init__(self, oid, prefix='fixup! ')
         if self.new_commitmsg:
             self.new_commitmsg = self.new_commitmsg.splitlines()[0]
 
@@ -995,7 +1090,10 @@ class OpenParentDir(OpenDefaultApp):
     def do(self):
         if not self.filenames:
             return
-        dirs = list(set(map(os.path.dirname, self.filenames)))
+        dirnames = list(set([os.path.dirname(x) for x in self.filenames]))
+        # os.path.dirname() can return an empty string so we fallback to
+        # the current directory
+        dirs = [(dirname or core.getcwd()) for dirname in dirnames]
         core.fork([self.launcher] + dirs)
 
 
@@ -1018,18 +1116,13 @@ class OpenRepo(Command):
 
     def do(self):
         git = self.model.git
-        old_worktree = git.worktree()
-        if not self.model.set_worktree(self.repo_path):
-            self.model.set_worktree(old_worktree)
-            return
-        new_worktree = git.worktree()
-        core.chdir(new_worktree)
-        self.model.set_directory(self.repo_path)
-        cfg = gitcfg.current()
-        cfg.reset()
-        fsmonitor.instance().stop()
-        fsmonitor.instance().start()
-        self.model.update_status()
+        old_repo = git.getcwd()
+        if self.model.set_worktree(self.repo_path):
+            fsmonitor.current().stop()
+            fsmonitor.current().start()
+            self.model.update_status()
+        else:
+            self.model.set_worktree(old_repo)
 
 
 class Clone(Command):
@@ -1057,7 +1150,7 @@ class Clone(Command):
             self.error_message = N_('Error: could not clone "%s"') % self.url
             self.error_details = (
                     (N_('git clone returned exit code %s') % status) +
-                     ((out+err) and ('\n\n' + out + err) or ''))
+                    ((out+err) and ('\n\n' + out + err) or ''))
 
         return self
 
@@ -1069,7 +1162,7 @@ def unix_path(path, is_win32=utils.is_win32):
     if is_win32():
         first = path[0]
         second = path[1]
-        if second == ':': # sanity check, this better be a Windows-style path
+        if second == ':':  # sanity check, this better be a Windows-style path
             unix_path = '/' + first + path[2:].replace('\\', '/')
 
     return unix_path
@@ -1145,7 +1238,12 @@ class Rebase(Command):
         with GitXBaseContext(
                 GIT_XBASE_TITLE=N_('Rebase onto %s') % upstream_title,
                 GIT_XBASE_ACTION=N_('Rebase')):
-            status, out, err = self.model.git.rebase(*args, **kwargs)
+                # XXX this blocks the user interface window for the duration of our
+                # git-xbase's invocation. would need to implement signals for
+                # QProcess and continue running the main thread. alternatively we
+                # could hide the main window while rebasing. that doesn't require
+                # as much effort.
+                status, out, err = self.model.git.rebase(*args, _no_win32_startupinfo=True, **kwargs)
         Interaction.log_status(status, out, err)
         self.model.update_status()
         return status, out, err
@@ -1214,7 +1312,7 @@ class Refresh(Command):
 
     def do(self):
         self.model.update_status(update_index=True)
-        fsmonitor.instance().refresh()
+        fsmonitor.current().refresh()
 
 
 class RevertEditsCommand(ConfirmAction):
@@ -1304,7 +1402,7 @@ class RunConfigAction(Command):
         self.model = main.model()
 
     def do(self):
-        for env in ('FILENAME', 'REVISION', 'ARGS'):
+        for env in ('ARGS', 'DIRNAME', 'FILENAME', 'REVISION'):
             try:
                 compat.unsetenv(env)
             except KeyError:
@@ -1328,7 +1426,9 @@ class RunConfigAction(Command):
                         N_('Please select a file'),
                         N_('"%s" requires a selected file.') % cmd)
                 return False
+            dirname = utils.dirname(filename, current_dir='.')
             compat.setenv('FILENAME', filename)
+            compat.setenv('DIRNAME', dirname)
 
         if opts.get('revprompt') or opts.get('argprompt'):
             while True:
@@ -1347,7 +1447,7 @@ class RunConfigAction(Command):
         elif opts.get('confirm'):
             title = os.path.expandvars(opts.get('title'))
             prompt = os.path.expandvars(opts.get('prompt'))
-            if Interaction.question(title, prompt):
+            if not Interaction.question(title, prompt):
                 return
         if rev:
             compat.setenv('REVISION', rev)
@@ -1376,9 +1476,10 @@ class RunConfigAction(Command):
 
 class SetDefaultRepo(Command):
 
-    def __init__(self, repo):
+    def __init__(self, repo, name):
         Command.__init__(self)
         self.repo = repo
+        self.name = name
 
     def do(self):
         gitcfg.current().set_user('cola.defaultrepo', self.repo)
@@ -1613,10 +1714,12 @@ class Tag(Command):
         log_msg = (N_('Tagging "%(revision)s" as "%(name)s"') %
                    dict(revision=self._revision, name=self._name))
         opts = {}
+        tmp_file = None
         try:
             if self._message:
-                opts['F'] = utils.tmp_filename('tag-message')
-                core.write(opts['F'], self._message)
+                tmp_file = utils.tmp_filename('tag-message')
+                opts['F'] = tmp_file
+                core.write(tmp_file, self._message)
 
             if self._sign:
                 log_msg += ' (%s)' % N_('GPG-signed')
@@ -1626,8 +1729,8 @@ class Tag(Command):
             status, output, err = self.model.git.tag(self._name,
                                                      self._revision, **opts)
         finally:
-            if 'F' in opts:
-                os.unlink(opts['F'])
+            if tmp_file:
+                core.unlink(tmp_file)
 
         if output:
             log_msg += '\n' + (N_('Output: %s') % output)
@@ -1814,3 +1917,76 @@ def do_cmd(cmd):
         msg, details = utils.format_exception(e)
         Interaction.critical(N_('Error'), message=msg, details=details)
         return None
+
+
+def difftool_run():
+    """Start a default difftool session"""
+    files = selection.selected_group()
+    if not files:
+        return
+    s = selection.selection()
+    model = main.model()
+    difftool_launch_with_head(files, bool(s.staged), model.head)
+
+
+def difftool_launch_with_head(filenames, staged, head):
+    """Launch difftool against the provided head"""
+    if head == 'HEAD':
+        left = None
+    else:
+        left = head
+    difftool_launch(left=left, staged=staged, paths=filenames)
+
+
+def difftool_launch(left=None, right=None, paths=None,
+                    staged=False, dir_diff=False,
+                    left_take_magic=False, left_take_parent=False):
+    """Launches 'git difftool' with given parameters
+
+    :param left: first argument to difftool
+    :param right: second argument to difftool_args
+    :param paths: paths to diff
+    :param staged: activate `git difftool --staged`
+    :param dir_diff: activate `git difftool --dir-diff`
+    :param left_take_magic: whether to append the magic ^! diff expression
+    :param left_take_parent: whether to append the first-parent ~ for diffing
+
+    """
+
+    difftool_args = ['git', 'difftool', '--no-prompt']
+    if staged:
+        difftool_args.append('--cached')
+    if dir_diff:
+        difftool_args.append('--dir-diff')
+
+    if left:
+        if left_take_parent or left_take_magic:
+            suffix = left_take_magic and '^!' or '~'
+            # Check root commit (no parents and thus cannot execute '~')
+            model = main.model()
+            git = model.git
+            status, out, err = git.rev_list(left, parents=True, n=1)
+            Interaction.log_status(status, out, err)
+            if status:
+                raise OSError('git rev-list command failed')
+
+            if len(out.split()) >= 2:
+                # Commit has a parent, so we can take its child as requested
+                left += suffix
+            else:
+                # No parent, assume it's the root commit, so we have to diff
+                # against the empty tree.  Git's empty tree is a built-in
+                # constant object name.
+                left = '4b825dc642cb6eb9a060e54bf8d69288fbee4904'
+                if not right and left_take_magic:
+                    right = left
+        difftool_args.append(left)
+
+    if right:
+        difftool_args.append(right)
+
+    if paths:
+        difftool_args.append('--')
+        difftool_args.extend(paths)
+
+    core.fork(difftool_args)
