@@ -135,7 +135,8 @@ class CommitMessageEditor(QtWidgets.QWidget):
         self.check_spelling_action = self.actions_menu.addAction(
                 N_('Check Spelling'))
         self.check_spelling_action.setCheckable(True)
-        self.check_spelling_action.setChecked(False)
+        self.check_spelling_action.setChecked(prefs.spellcheck())
+        self.toggle_check_spelling(prefs.spellcheck())
 
         # Line wrapping
         self.autowrap_action = self.actions_menu.addAction(
@@ -199,8 +200,6 @@ class CommitMessageEditor(QtWidgets.QWidget):
         self.description.leave.connect(self.focus_summary)
         self.updated.connect(self.refresh)
 
-        self.summary.hint.enable(True)
-        self.description.hint.enable(True)
         self.commit_group.setEnabled(False)
 
         self.set_tabwidth(prefs.tabwidth())
@@ -219,6 +218,16 @@ class CommitMessageEditor(QtWidgets.QWidget):
         self.setFont(qtutils.diff_font())
         self.setFocusProxy(self.summary)
 
+        gitcfg.current().add_observer(gitcfg.current().message_user_config_changed, self.on_user_setting_changed)
+
+    def on_user_setting_changed(self, key, value):
+        if key != prefs.SPELL_CHECK:
+            return
+        if self.check_spelling_action.isChecked() == value:
+            return
+        self.check_spelling_action.setChecked(value)
+        self.toggle_check_spelling(value)
+
     def refresh(self):
         enabled = self.model.stageable() or self.model.unstageable()
         if self.model.stageable():
@@ -360,16 +369,10 @@ class CommitMessageEditor(QtWidgets.QWidget):
         focus_description = not description
 
         # Update summary
-        if not summary and not self.summary.hasFocus():
-            self.summary.hint.enable(True)
-        else:
-            self.summary.set_value(summary, block=True)
+        self.summary.set_value(summary, block=True)
 
         # Update description
-        if not description and not self.description.hasFocus():
-            self.description.hint.enable(True)
-        else:
-            self.description.set_value(description, block=True)
+        self.description.set_value(description, block=True)
 
         # Update text color
         self.refresh_palettes()
@@ -428,7 +431,10 @@ class CommitMessageEditor(QtWidgets.QWidget):
 
         msg = self.commit_message(raw=False)
 
-        if not self.model.staged:
+        # we either need to have something staged, or be merging
+        # (if there was a merge conflict resolved, there may not be anything to stage,
+        #  but we still need to commit to complete the merge)
+        if not (self.model.staged or self.model.is_merging):
             error_msg = N_(
                 'No changes to commit.\n\n'
                 'You must stage at least 1 file before you can commit.')
@@ -514,6 +520,8 @@ class CommitMessageEditor(QtWidgets.QWidget):
     def toggle_check_spelling(self, enabled):
         spellcheck = self.description.spellcheck
 
+        if gitcfg.current().get_user(prefs.SPELL_CHECK) != enabled:
+            gitcfg.current().set_user(prefs.SPELL_CHECK, enabled)
         if enabled and not self.spellcheck_initialized:
             # Add our name to the dictionary
             self.spellcheck_initialized = True
@@ -545,7 +553,7 @@ class CommitSummaryLineEdit(HintedLineEdit):
 
     def __init__(self, parent=None):
         hint = N_('Commit summary')
-        HintedLineEdit.__init__(self, hint, parent)
+        HintedLineEdit.__init__(self, hint, parent=parent)
         self.extra_actions = []
 
         comment_char = prefs.comment_char()
