@@ -39,7 +39,7 @@ class CompletionLineEdit(text.HintedLineEdit):
     }
 
     def __init__(self, model_factory, hint='', parent=None):
-        text.HintedLineEdit.__init__(self, hint=hint, parent=parent)
+        text.HintedLineEdit.__init__(self, hint, parent=parent)
         # Tracks when the completion popup was active during key events
         self._was_visible = False
         # The most recently selected completion item
@@ -266,8 +266,9 @@ class GatherCompletionsThread(QtCore.QThread):
 class HighlightDelegate(QtWidgets.QStyledItemDelegate):
     """A delegate used for auto-completion to give formatted completion"""
 
-    def __init__(self, parent=None):
+    def __init__(self, parent):
         QtWidgets.QStyledItemDelegate.__init__(self, parent)
+        self.widget = parent
         self.highlight_text = ''
         self.case_sensitive = False
 
@@ -320,7 +321,7 @@ class HighlightDelegate(QtWidgets.QStyledItemDelegate):
 
         # translate the painter to where the text is drawn
         item_text = QtWidgets.QStyle.SE_ItemViewItemText
-        rect = style.subElementRect(item_text, params)
+        rect = style.subElementRect(item_text, params, self.widget)
         painter.save()
 
         start = rect.topLeft() + QtCore.QPoint(defs.margin, 0)
@@ -628,12 +629,12 @@ class GitLogCompletionModel(GitRefCompletionModel):
         return (refs, paths, dirs)
 
 
-def bind_lineedit(model):
+def bind_lineedit(model, hint=''):
     """Create a line edit bound against a specific model"""
 
     class BoundLineEdit(CompletionLineEdit):
 
-        def __init__(self, hint='', parent=None):
+        def __init__(self, hint=hint, parent=None):
             CompletionLineEdit.__init__(self, model,
                                         hint=hint, parent=parent)
 
@@ -641,13 +642,16 @@ def bind_lineedit(model):
 
 
 # Concrete classes
-GitLogLineEdit = bind_lineedit(GitLogCompletionModel)
-GitRefLineEdit = bind_lineedit(GitRefCompletionModel)
-GitPotentialBranchLineEdit = bind_lineedit(GitPotentialBranchCompletionModel)
-GitBranchLineEdit = bind_lineedit(GitBranchCompletionModel)
-GitRemoteBranchLineEdit = bind_lineedit(GitRemoteBranchCompletionModel)
-GitStatusFilterLineEdit = bind_lineedit(GitStatusFilterCompletionModel)
-GitTrackedLineEdit = bind_lineedit(GitTrackedCompletionModel)
+GitLogLineEdit = bind_lineedit(GitLogCompletionModel, hint='<ref>')
+GitRefLineEdit = bind_lineedit(GitRefCompletionModel, hint='<ref>')
+GitPotentialBranchLineEdit = bind_lineedit(GitPotentialBranchCompletionModel,
+                                           hint='<branch>')
+GitBranchLineEdit = bind_lineedit(GitBranchCompletionModel, hint='<branch>')
+GitRemoteBranchLineEdit = bind_lineedit(GitRemoteBranchCompletionModel,
+                                        hint='<remote-branch>')
+GitStatusFilterLineEdit = bind_lineedit(GitStatusFilterCompletionModel,
+                                        hint='<path>')
+GitTrackedLineEdit = bind_lineedit(GitTrackedCompletionModel, hint='<path>')
 
 
 class GitDialog(QtWidgets.QDialog):
