@@ -1,17 +1,17 @@
 from __future__ import division, absolute_import, unicode_literals
 
-from PyQt4 import QtGui
-from PyQt4 import QtCore
-from PyQt4.QtCore import Qt
+from qtpy import QtCore
+from qtpy import QtWidgets
+from qtpy.QtCore import Qt
 
-from cola import cmds
-from cola import icons
-from cola import qtutils
-from cola.i18n import N_
-from cola.widgets import defs
-from cola.widgets import completion
-from cola.widgets import standard
-from cola.widgets import text
+from .. import cmds
+from .. import icons
+from .. import qtutils
+from ..i18n import N_
+from . import defs
+from . import completion
+from . import standard
+from . import text
 
 
 def new_create_tag(name='', ref='', sign=False, settings=None, parent=None):
@@ -31,7 +31,6 @@ def create_tag(name='', ref='', sign=False, settings=None):
     return view
 
 
-
 class TagOptions(object):
     """Simple data container for the CreateTag dialog."""
 
@@ -48,35 +47,33 @@ class CreateTag(standard.Dialog):
 
         self.opts = opts
 
-        self.setAttribute(Qt.WA_MacMetalStyle)
         self.setWindowTitle(N_('Create Tag'))
         if parent is not None:
             self.setWindowModality(QtCore.Qt.WindowModal)
 
         # Tag label
-        self.tag_name_label = QtGui.QLabel(self)
+        self.tag_name_label = QtWidgets.QLabel(self)
         self.tag_name_label.setText(N_('Name'))
 
-        self.tag_name = text.HintedLineEdit(N_('vX.Y.Z'), self)
+        self.tag_name = text.HintedLineEdit(N_('vX.Y.Z'), parent=self)
         self.tag_name.set_value(opts.name)
         self.tag_name.setToolTip(N_('Specifies the tag name'))
 
         # Sign Tag
-        self.sign_label = QtGui.QLabel(self)
+        self.sign_label = QtWidgets.QLabel(self)
         self.sign_label.setText(N_('Sign Tag'))
 
         tooltip = N_('Whether to sign the tag (git tag -s)')
         self.sign_tag = qtutils.checkbox(checked=True, tooltip=tooltip)
 
         # Tag message
-        self.tag_msg_label = QtGui.QLabel(self)
+        self.tag_msg_label = QtWidgets.QLabel(self)
         self.tag_msg_label.setText(N_('Message'))
 
-        self.tag_msg = text.HintedTextEdit(N_('Tag message...'), self)
+        self.tag_msg = text.HintedPlainTextEdit(N_('Tag message...'), self)
         self.tag_msg.setToolTip(N_('Specifies the tag message'))
-        self.tag_msg.hint.enable(True)
         # Revision
-        self.rev_label = QtGui.QLabel(self)
+        self.rev_label = QtWidgets.QLabel(self)
         self.rev_label.setText(N_('Revision'))
 
         self.revision = completion.GitRefLineEdit()
@@ -84,7 +81,8 @@ class CreateTag(standard.Dialog):
         self.revision.setToolTip(N_('Specifies the SHA-1 to tag'))
         # Buttons
         self.create_button = qtutils.create_button(text=N_('Create Tag'),
-                                                   icon=icons.tag())
+                                                   icon=icons.tag(),
+                                                   default=True)
         self.close_button = qtutils.close_button()
 
         # Form layout for inputs
@@ -94,9 +92,10 @@ class CreateTag(standard.Dialog):
                                          (self.rev_label, self.revision),
                                          (self.sign_label, self.sign_tag))
 
-        self.button_layout = qtutils.hbox(defs.no_margin, defs.spacing,
-                                          qtutils.STRETCH, self.create_button,
-                                          self.close_button)
+        self.button_layout = qtutils.hbox(defs.no_margin, defs.button_spacing,
+                                          self.close_button,
+                                          qtutils.STRETCH,
+                                          self.create_button)
 
         self.main_layt = qtutils.vbox(defs.margin, defs.spacing,
                                       self.input_layout, self.button_layout)
@@ -105,8 +104,7 @@ class CreateTag(standard.Dialog):
         qtutils.connect_button(self.close_button, self.close)
         qtutils.connect_button(self.create_button, self.create_tag)
 
-        if not self.restore_state(settings=settings):
-            self.resize(defs.scale(720), defs.scale(210))
+        self.init_state(settings, self.resize, defs.scale(720), defs.scale(210))
 
     def create_tag(self):
         """Verifies inputs and emits a notifier tag message."""
@@ -147,4 +145,3 @@ class CreateTag(standard.Dialog):
             qtutils.critical(N_('Error: could not create tag "%s"') % tag_name,
                              (N_('git tag returned exit code %s') % status) +
                              ((output+err) and ('\n\n' + output + err) or ''))
-
