@@ -57,10 +57,10 @@ class WidgetMixin(object):
         return self.__class__.__name__.lower()
 
     def save_state(self, settings=None):
-        if settings is None:
-            settings = Settings()
-            settings.load()
         if gitcfg.current().get('cola.savewindowsettings', True):
+            if settings is None:
+                settings = Settings()
+                settings.load()
             settings.save_gui_state(self)
 
     def resizeEvent(self, event):
@@ -153,22 +153,32 @@ class WidgetMixin(object):
 
         return ret
 
-    def save_settings(self):
-        settings = Settings()
-        settings.load()
-        settings.add_recent(core.getcwd())
+    def save_settings(self, settings=None):
         return self.save_state(settings=settings)
 
     def closeEvent(self, event):
         self.save_settings()
         self.Base.closeEvent(self, event)
 
+    def init_size(self, parent=None, settings=None, width=0, height=0):
+        if not width:
+            width = defs.dialog_w
+        if not height:
+            height = defs.dialog_h
+        self.init_state(settings,
+                        self.resize_to_parent, parent, width, height)
+
     def init_state(self, settings, callback, *args, **kwargs):
         """Restore saved settings or set the initial location"""
         if not self.restore_state(settings=settings):
             callback(*args, **kwargs)
             self.center()
 
+    def resize_to_parent(self, parent, w, h):
+        """Set the initial size of the widget"""
+        width, height = qtutils.default_size(parent, w, h)
+        self.resize(width, height)
+
 
 class MainWindowMixin(WidgetMixin):
 
@@ -188,6 +198,13 @@ class MainWindowMixin(WidgetMixin):
         state['windowstate'] = windowstate.toBase64().data().decode('ascii')
         return state
 
+    def save_settings(self, settings=None):
+        if settings is None:
+            settings = Settings()
+            settings.load()
+            settings.add_recent(core.getcwd())
+        return WidgetMixin.save_settings(self, settings=settings)
+
     def apply_state(self, state):
         result = WidgetMixin.apply_state(self, state)
         windowstate = state.get('windowstate', None)
@@ -361,6 +378,22 @@ class TreeMixin(object):
                 item = widget.model().itemFromIndex(index)
         return item
 
+    def column_widths(self):
+        """Return the tree's column widths"""
+        widget = self.widget
+        count = widget.header().count()
+        return [widget.columnWidth(i) for i in range(count)]
+
+    def set_column_widths(self, widths):
+        """Set the tree's column widths"""
+        if widths:
+            widget = self.widget
+            count = widget.header().count()
+            if len(widths) > count:
+                widths = widths[:count]
+            for idx, value in enumerate(widths):
+                widget.setColumnWidth(idx, value)
+
 
 class DraggableTreeMixin(TreeMixin):
     """A tree widget with internal drag+drop reordering of rows
@@ -436,16 +469,26 @@ class Widget(WidgetMixin, QtWidgets.QWidget):
 class Dialog(WidgetMixin, QtWidgets.QDialog):
     Base = QtWidgets.QDialog
 
-    def __init__(self, parent=None, save_settings=False):
+    def __init__(self, parent=None):
         QtWidgets.QDialog.__init__(self, parent)
         WidgetMixin.__init__(self)
-        self._save_settings = save_settings
+
+    def accept(self):
+        self.save_settings()
+        return self.Base.accept(self)
 
     def reject(self):
-        if self._save_settings:
-            self.save_settings()
+        self.save_settings()
         return self.Base.reject(self)
 
+    def close(self):
+        """save_settings() is handled by accept() and reject()"""
+        self.Base.close(self)
+
+    def closeEvent(self, event):
+        """save_settings() is handled by accept() and reject()"""
+        self.Base.closeEvent(self, event)
+
 
 class MainWindow(MainWindowMixin, QtWidgets.QMainWindow):
     Base = QtWidgets.QMainWindow
@@ -454,32 +497,6 @@ class MainWindow(MainWindowMixin, QtWidgets.QMainWindow):
         QtWidgets.QMainWindow.__init__(self, parent)
         MainWindowMixin.__init__(self)
 
-        palette = self.palette()
-        window = palette.color(QtGui.QPalette.Window)
-        highlight = palette.color(QtGui.QPalette.Highlight)
-
-        window_rgb = ('rgb(%d, %d, %d)'
-                 % (window.red(),
-                    window.blue(),
-                    window.green()))
-        highlight_rgb = ('rgb(%d, %d, %d)'
-                 % (highlight.red(),
-                    highlight.blue(),
-                    highlight.green()))
-
-        self.setStyleSheet("""
-            QMainWindow::separator {
-                background: %(window_rgb)s;
-                width: %(separator)spx;
-                height: %(separator)spx;
-            }
-            QMainWindow::separator:hover {
-                background: %(highlight_rgb)s;
-            }
-            """ % dict(separator=defs.separator,
-                       window_rgb=window_rgb,
-                       highlight_rgb=highlight_rgb))
-
 
 class TreeView(QtWidgets.QTreeView):
     Mixin = TreeMixin
@@ -507,6 +524,12 @@ class TreeView(QtWidgets.QTreeView):
     def items(self):
         return self._mixin.items()
 
+    def column_widths(self):
+        return self._mixin.column_widths()
+
+    def set_column_widths(self, widths):
+        return self._mixin.set_column_widths(widths)
+
 
 class TreeWidget(QtWidgets.QTreeWidget):
     Mixin = TreeMixin
@@ -534,6 +557,12 @@ class TreeWidget(QtWidgets.QTreeWidget):
     def items(self):
         return self._mixin.items()
 
+    def column_widths(self):
+        return self._mixin.column_widths()
+
+    def set_column_widths(self, widths):
+        return self._mixin.set_column_widths(widths)
+
 
 class DraggableTreeWidget(TreeWidget):
     Mixin = DraggableTreeMixin
